﻿namespace WarmlyLocomotive;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningWarmlyLocomotive
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityWarmlyLocomotive? EntityWarmlyLocomotive { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningLocomotiveWidth = 150;
    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningLocomotiveHeight = 100;
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>

    public void Init(int speed, double weight, Color bodyColor, Color
    additionalColor, bool tube, bool fuelTank)
    {
        EntityWarmlyLocomotive = new EntityWarmlyLocomotive();
        EntityWarmlyLocomotive.Init(speed, weight, bodyColor, additionalColor,
        tube, fuelTank);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (width >= _drawningLocomotiveWidth && height >= _drawningLocomotiveHeight) {
            _pictureWidth = width;
            _pictureHeight = height;
            return true;
        }
        return false;
        
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if ((y + _drawningLocomotiveHeight) > _pictureHeight)
        {
            _startPosY = y - _drawningLocomotiveHeight;
        }
        else
        {
            _startPosY = y;
        }

        if ((x + _drawningLocomotiveWidth) > _pictureWidth)
        {
            _startPosX = x - _drawningLocomotiveWidth;
        }
        else
        {
            _startPosX = x;
        }

    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityWarmlyLocomotive == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityWarmlyLocomotive.Step > 0)
                {
                    _startPosX -= (int)EntityWarmlyLocomotive.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityWarmlyLocomotive.Step > 0)
                {
                    _startPosY -= (int)EntityWarmlyLocomotive.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityWarmlyLocomotive.Step + _drawningLocomotiveWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityWarmlyLocomotive.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityWarmlyLocomotive.Step + _drawningLocomotiveHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityWarmlyLocomotive.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityWarmlyLocomotive == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityWarmlyLocomotive.AdditionalColor);

        //труба
        if (EntityWarmlyLocomotive.Tube)
        {
            g.DrawRectangle(pen, _startPosX.Value + 40, _startPosY.Value, 20, 30);
            g.FillRectangle(additionalBrush, _startPosX.Value + 40, _startPosY.Value, 20, 40);
        }

        //локомотив
        Brush br = new SolidBrush(EntityWarmlyLocomotive.BodyColor);
        Brush blBr = new SolidBrush(Color.Black);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 60, 140, 20);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 60, 140, 20);
        Point point1 = new Point(_startPosX.Value, _startPosY.Value + 60);
        Point point2 = new Point(_startPosX.Value + 20, _startPosY.Value + 30);
        Point point3 = new Point(_startPosX.Value + 140, _startPosY.Value + 30);
        Point point4 = new Point(_startPosX.Value + 140, _startPosY.Value + 60);
        Point[] body ={point1, point2, point3, point4};
        g.DrawPolygon(pen, body);
        g.FillPolygon(br, body);
        g.DrawRectangle(pen, _startPosX.Value + 140, _startPosY.Value + 40, 10, 40);
        g.FillRectangle(br, _startPosX.Value + 140, _startPosY.Value + 40, 10, 40);
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 80, 20, 20);
        g.FillEllipse(blBr, _startPosX.Value, _startPosY.Value + 80, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 80, 20, 20);
        g.FillEllipse(blBr, _startPosX.Value + 30, _startPosY.Value + 80, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 90, _startPosY.Value + 80, 20, 20);
        g.FillEllipse(blBr, _startPosX.Value + 90, _startPosY.Value + 80, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 120, _startPosY.Value + 80, 20, 20);
        g.FillEllipse(blBr, _startPosX.Value + 120, _startPosY.Value + 80, 20, 20);

        // отсек для топлива
        if (EntityWarmlyLocomotive.FuelTank)
        {
            g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 40, 50, 40);
            g.FillRectangle(additionalBrush, _startPosX.Value + 80, _startPosY.Value + 40, 50, 40);
        }

    }
}